#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <usb.h>

char	cmd1[] = { 8, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0 },
  	cmd2[] = { 8, 0, 0, 0, 6, 1, 0, 0, 0, 0, 0, 0 },
  	cmd3[] = { 8, 0, 0, 0, 0x94, 0, 0, 0, 0, 0, 0, 0 },
  	cmd4[] = { 8, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0 },
  	cmd5[] = { 8, 0, 0, 0, 3, 1, 0, 0, 0, 0, 0, 0 },
  	cmd6[] = { 8, 0, 0, 0, 8, 1, 0, 0, 0xe, 0, 0, 0 },
  	cmd7[] = { 8, 0, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0 },
	cmd8[] = { 0xc, 0, 0, 0 },
  	cmd9[] = { 8, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0 },
  	cmd10[] = { 8, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0 };

int get_listing( usb_dev_handle *hndl, char *&list ) {
  int rc, files_count, list_len;
  char buf[0x20];

  rc = usb_bulk_write( hndl, 2, cmd1, sizeof(cmd1), 1000 );
  if ( rc == -1 ) {
  	printf( "mito.cc %s\n", usb_strerror() );
	return -1;
  }
  usb_bulk_write( hndl, 2, cmd2, sizeof(cmd2), 1000 );
  rc = usb_bulk_read( hndl, 0x83, buf, 8, 100 );
  if ( rc == -1 ) {
  	printf( "mito.cc %s\n", usb_strerror() );
	return -1;
  }
  usb_bulk_write( hndl, 2, cmd3, sizeof(cmd3), 1000 );
  usb_bulk_write( hndl, 2, cmd4, sizeof(cmd4), 1000 );
  usb_bulk_write( hndl, 2, cmd5, sizeof(cmd5), 1000 );
  usb_bulk_read( hndl, 0x83, (char*) &files_count, 4, 1000 ); // files count
  
  list_len = files_count * 13 + 1;
  list = (char*) malloc( list_len );
  *(int*)&cmd6[8] = list_len;
  usb_bulk_write( hndl, 2, cmd6, sizeof(cmd6), 1000 );
  usb_bulk_read( hndl, 0x83, list, list_len, 1000 ); // file names
  
  return files_count;
}

int get_filesize( usb_dev_handle *hndl, char *filename ) {
  char buf[0x20];
  int rc, filesize;

  usb_bulk_write( hndl, 2, cmd7, sizeof(cmd7), 1000 );
  usb_bulk_read( hndl, 0x83, buf, 4, 1000 );
  memcpy( buf, cmd8, 4 );
  memcpy( buf + 4, filename, 12 );
  rc = usb_bulk_write( hndl, 2, buf, 0x10, 1000 );		// file name
  if ( rc == -1 ) {
  	printf( "mito.cc write filename - %s\n", usb_strerror );
	return -1;
  }
  rc = usb_bulk_read( hndl, 0x83, (char*) &filesize, 4, 1000 );	// file size
  if ( rc == -1 ) {
  	printf( "mito.cc read filesize - %s\n", usb_strerror );
	return -1;
  }

  return filesize;
}

int lookup_file( char *&list, int ent_count, char *filename ) {
  if ( strlen( filename ) != 12 ) return 0;
  for ( int i = 0; i < ent_count; i++ )
  	if ( !memcmp( list + i * 13, filename, 12 ) ) return 1;
  return 0;
}

void mito_list( usb_dev_handle *hndl ) {
  int files_count, filesize;
  char *list;
  
  files_count = get_listing( hndl, list );
  if ( files_count == -1 ) return;

  for ( int i = 0; i < files_count; i++ ) {
  	list[i * 13 + 12] = 0;
	filesize = get_filesize( hndl, list + i * 13 );
	printf( "%s (%d bytes)\n", list + i * 13, filesize );
  }
}

void mito_del( usb_dev_handle *hndl, char *filename ) {
  char buf[0x20], *list;
  int rc, files_count;
  
  files_count = get_listing( hndl, list );
  if ( files_count == -1 ) return;

  if ( !lookup_file( list, files_count, filename ) ) {
  	puts( "mito.cc file not found" );
	return;
  }

  usb_bulk_write( hndl, 2, cmd1, sizeof(cmd1), 1000 );
  usb_bulk_write( hndl, 2, cmd2, sizeof(cmd2), 1000 );
  usb_bulk_read( hndl, 0x83, buf, 8, 100 );
  usb_bulk_write( hndl, 2, cmd3, sizeof(cmd3), 1000 );
  usb_bulk_write( hndl, 2, cmd4, sizeof(cmd4), 1000 );
  usb_bulk_write( hndl, 2, cmd10, sizeof(cmd10), 1000 );
  
  memcpy( buf, cmd8, 4 );
  memcpy( buf + 4, filename, 12 );
  rc = usb_bulk_write( hndl, 2, buf, 0x10, 1000 );
  if ( rc == -1 ) {
  	printf( "mito.cc del file on writing - %s\n", usb_strerror );
	return;
  }
  usb_bulk_read( hndl, 0x83, buf, 4, 1000 );
}

void mito_get( usb_dev_handle *hndl, char *filename ) {
  int filesize, listsize, rc, fd;
  char buf[0x20], *databuf, *list;

  listsize = get_listing( hndl, list );
  if ( listsize == -1 ) return;
  
  if ( !lookup_file( list, listsize, filename ) ) {
  	puts( "mito.cc file not found" );
	return;
  }
  
  filesize = get_filesize( hndl, filename );
  if ( filesize <= 0 ) return;
  
  printf( "downloading %s (%d bytes)\n", filename, filesize );
  
  usb_bulk_write( hndl, 2, cmd2, sizeof(cmd2), 1000 );
  usb_bulk_read( hndl, 0x83, buf, 8, 1000 );
  usb_bulk_write( hndl, 2, cmd9, sizeof(cmd9), 1000 );
  usb_bulk_read( hndl, 0x83, buf, 4, 1000 );
  memcpy( buf, cmd8, 4 );
  memcpy( buf + 4, filename, 12 );
  usb_bulk_write( hndl, 2, buf, 0x10, 1000 );		// file name
  databuf = (char*) malloc( filesize );
  rc = usb_bulk_read( hndl, 0x83, databuf, filesize, 100000 );
  if ( rc == -1 ) {
  	printf( "mito.cc readfile %s\n", usb_strerror() );
	return;
  }
 
  fd = open( filename, O_WRONLY | O_CREAT, 0644 );
  if ( fd == -1 ) {
  	perror( "mito.cc open" );
	return;
  }
  write( fd, databuf, filesize );
  close( fd );
}

int main( int argc, char **argv ) {
  struct usb_bus *bus;
  struct usb_device *dev;
  struct usb_dev_handle *hndl;

  if ( argc == 2 && !strcmp( argv[1], "list" ) ) {
  }
  else if ( argc == 3 && !strcmp( argv[1], "get" ) ) {
  }
  else if ( argc == 3 && !strcmp( argv[1], "del" ) ) {
  }
  else {
  	printf( "Syntax:\n" );
	printf( "mito list      - lists files that are in camera memory\n" );
	printf( "mito get file  - download file from camera\n" );
	printf( "mito del file  - remove file from camera memory\n" );
	return 1;
  }

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for ( bus = usb_get_busses(); bus; bus = bus->next ) {
  	for ( dev = bus->devices; dev; dev = dev->next )
		if ( dev->descriptor.idVendor == 0x919 &&
		     dev->descriptor.idProduct == 0x100 )
			goto device_found;
  }
  puts( "Camera Mito DS1300F is not connected" );
  return 1;
  
device_found: 
  hndl = usb_open( dev );
  usb_claim_interface( hndl, 0 );

  if ( !strcmp( argv[1], "list" ) ) mito_list( hndl );
  else if ( !strcmp( argv[1], "get" ) ) mito_get( hndl, argv[2] );
  else mito_del( hndl, argv[2] );

  usb_release_interface( hndl, 0 );
  usb_close( hndl );
  
  return 0;
}
